import java.sql.*;
import java.util.Properties;
import java.util.*;

class DBAccess {

  private Connection conn;

  private boolean debugApplet = true;

  public static final int TRUE_FALSE = 0;
  public static final int MULTIPLE_CHOICE = 1;
  public static final int FILL_IN = 2;
  public static final int CATEGORY = 3;
  public static final int MATCHING = 4;
  public static final int SINGLE_CHOICE = 5;

  public static final int YesNo = 1;
  public static final int True = 1;

  public static final String TRUE1 = "TRUE";
  public static final String TRUE2 = "True";
  public static final String TRUE3 = "true";

  public static final String YES1 = "YES";
  public static final String YES2 = "Yes";
  public static final String YES3 = "yes";

 /*******************************************************
  *
  *  Method Name : printLine
  *  Function    : print out if debugApplet is true.
  *
  *******************************************************/
  public void printLine(String printData) {
    if ( debugApplet ) {
      System.out.println("Voter Database : " + printData);
    }
  }

 /*******************************************************
  *
  *  Method Name : open
  *  Function    : make a connection to the database
  *
  *******************************************************/
  public void open() {
    try {
      Class.forName("oracle.jdbc.driver.OracleDriver");
      conn = DriverManager.getConnection
	("jdbc:oracle:thin:@carver.npac.syr.edu:1521:europe", "dongmin", "dongmin");
	//("jdbc:oracle:thin:@carver.npac.syr.edu:1521:europe", "cy99sso", "cy99sso");
      printLine("Connection Open");
    }
    catch (Exception e) {
      printLine("Error connection: " + e);
    }
  }

 /*******************************************************
  *
  *  Method Name : close
  *  Function    : close the connection to the database
  *
  *******************************************************/
  public void close() {
    try {
      conn.close();
      printLine("Connection Closed");
    }
    catch (Exception e) {
      printLine("Error in closing dbase");
    }
  }

  //
  // retreive data
  //
 /*******************************************************
  *
  *  Method Name : getAuthorList
  *  Function    : retreive the list of authors
  *
  *******************************************************/
  public Vector getAuthorList() {

    String query = null;
    Vector toVector = new Vector();

    printLine("author list");
    query = "select authorName from authorTable order by authorName";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getCourseList
  *  Function    : retreive the list of courses
  *
  *******************************************************/
  public Vector getCourseList(String authorName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;

    authorID = getAuthorID(authorName);

    printLine("course list");
    query = "select a.courseName from courseTable a, authorCourse b where " +
            "a.courseID = b.courseID and "   +
            "b.authorID='" + authorID + "' " +
            "order by a.courseName";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getLectureList
  *  Function    : retreive the list of lectures
  *
  *******************************************************/
  public Vector getLectureList(String authorName, String courseName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;
    int courseID;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);

    printLine("lecture list");
    query = "select lectureName from lectureTable where " +
            "authorID='" + authorID + "' and " +
            "courseID='" + courseID + "' " +
            "order by lectureName";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getQuizList
  *  Function    : retreive the list of quiz
  *
  *******************************************************/
  public Vector getQuizList(String authorName, String courseName, 
			    String lectureName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;
    int courseID;
    int lectureID;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);

    printLine("quiz list");
    query = "select quizName from quizTable where " +
            "authorID='"  + authorID  + "' and " +
            "courseID='"  + courseID  + "' and " +
            "lectureID='" + lectureID + "' "     +
            "order by quizName";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getQuestionList
  *  Function    : retreive the list of questions
  *
  *******************************************************/
  public Vector getQuestionList(String authorName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;

    authorID = getAuthorID(authorName);

    printLine("question list");
    query = "select question from questionTable where " +
            "authorID='" + authorID + "' " +
            "order by question";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getQuestionPerQuiz
  *  Function    : retreive the questions in the quiz 
  *
  *******************************************************/
  public Vector getQuestionPerQuiz(String authorName, String courseName, String lectureName, String quizName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;
    int courseID;
    int lectureID;
    int quizID;

    authorID  = getAuthorID(authorName);
    courseID  = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);
    quizID    = getQuizID(authorName, courseName, lectureName, quizName);

    printLine("qustions in quiz");

    query = "select b.question from quizQuestion a, questionTable b where " +
            "a.questionID = b.questionID and " +
            "a.quizID='" + quizID + "' " +
            "order by b.question";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getQuiz
  *  Function    : retreive the questions in the quiz 
  *
  *******************************************************/
  public Vector getQuiz(String authorName, String courseName, String lectureName, String quizName) {

    String query = null;
    Vector toVector = new Vector();
    int authorID;
    int courseID;
    int lectureID;
    int quizID;

    authorID  = getAuthorID(authorName);
    courseID  = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);
    quizID    = getQuizID(authorName, courseName, lectureName, quizName);

    printLine("qustions in quiz");

    query = "select b.question from quizQuestion a, questionTable b where " +
            "a.questionID = b.questionID and " +
            "a.quizID='" + quizID + "' " +
            "order by b.question";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      while (rs.next()) {
	toVector.addElement(rs.getString(1));
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    return toVector;
  }

 /*******************************************************
  *
  *  Method Name : getQuestion
  *  Function    : retreive the question
  *
  *******************************************************/
  public AbstractQuestion getQuestion(String authorName, 
				      String question) {

    String query = null;
    String relatedTable = null;
    int authorID;
    int questionID;

    // common information
    int    questionType = 0;
    int    answerTypeNumber = 0;
    int    timeLimit_min = 0;
    int    timeLimit_sec = 0;
    int    totalTry = 0;
    int    correctAnswer = 0;

    authorID = getAuthorID(authorName);
    questionID = getQuestionID(authorName, question);

    query = "select question, questionType, answerTypeNumber, timeLimit_min, timeLimit_sec, totalTry, correctAnswer, relatedTable from questionTable where " +
            "questionID='" + questionID + "' and " +
            "authorID='" + authorID + "'";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( !rs.next() ) {
	return null;
      }
      // question
      question = rs.getString(1);
      // question type
      questionType = java.lang.Integer.parseInt(rs.getString(2));
      // answer type or no. of answers
      answerTypeNumber = java.lang.Integer.parseInt(rs.getString(3));
      // time limit (minutes)
      timeLimit_min = java.lang.Integer.parseInt(rs.getString(4));
      // time limit (seconds)
      timeLimit_sec = java.lang.Integer.parseInt(rs.getString(5));
      // total try
      totalTry = java.lang.Integer.parseInt(rs.getString(6));
      // no. of correct answer
      correctAnswer = java.lang.Integer.parseInt(rs.getString(7));

      relatedTable = rs.getString(8);

      rs.close();

      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    // now the answers are stored in answerVector
    QuestionFactory questionFactory = new QuestionFactory();

    switch(questionType) {
    //
    // true false
    case QuestionFactory.TRUEFALSE     : {
      TrueFalse tf = (TrueFalse) questionFactory.create(QuestionFactory.TRUEFALSE);
      getValues(tf, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) tf;
    }
    //
    // multiple choice
    case QuestionFactory.MULTIPLECHOICE: {
      MultipleChoice mc = (MultipleChoice) questionFactory.create(QuestionFactory.MULTIPLECHOICE);
      getValues(mc, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) mc;                
    }
    //
    // fill in
    case QuestionFactory.FILLINTHEBLANK: {
      FillInTheBlank fb = (FillInTheBlank) questionFactory.create(QuestionFactory.FILLINTHEBLANK);
      getValues(fb, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) fb;                
    }
    //
    // category
    case QuestionFactory.CATEGORY      : {
      Category c = (Category) questionFactory.create(QuestionFactory.CATEGORY);
      getValues(c, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) c;               
    }
    //
    // matching
    case QuestionFactory.MATCHING      : {
      Matching m = (Matching) questionFactory.create(QuestionFactory.MATCHING);
      getValues(m, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) m;                  
    }
    //
    // single choice (selection)
    case QuestionFactory.SELECTION     : {
      Selection s = (Selection) questionFactory.create(QuestionFactory.SELECTION);
      getValues(s, questionID,
		 question, answerTypeNumber, 
		 timeLimit_min, timeLimit_sec, 
		 totalTry, correctAnswer, 
		 relatedTable);
      return (AbstractQuestion) s;                  
    }
    default: return null;
    }
  }

 /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for true-false
  *
  *******************************************************/
  public void getValues(TrueFalse tf, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    boolean answer = true;

    String query = "select answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' ";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      if ( !rs.next() ) {
	return;
      }

      if ( java.lang.Integer.parseInt(rs.getString(1)) == True ) {
	answer = true;
      }
      else {
	answer = false;
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase getVales(true false)");
      e.printStackTrace();
    }

    tf.setText(question);
    if ( answerTypeNumber == YesNo ) {
      tf.setTrueText("Yes");
      tf.setFalseText("No");
    }
    else {
      tf.setTrueText("True");
      tf.setFalseText("False");
    }

    tf.setCorrectAnswer(answer);
    tf.setTime(timeLimit_min, timeLimit_sec);
  }

 /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for multiple choice
  *
  *******************************************************/
  public void getValues(MultipleChoice mc, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    mc.setText(question);
    String query = "select list, answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' ";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      while (rs.next()) {
	mc.setOption(rs.getString(1), 'm');
	if ( java.lang.Integer.parseInt(rs.getString(2)) == True ) {
	  mc.addCorrectAnswer(rs.getString(1));
	}
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    mc.setTime(timeLimit_min, timeLimit_sec);
  }

  /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for fill in
  *
  *******************************************************/
  public void getValues(FillInTheBlank fb, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    Vector tmpAnswer = new Vector();

    fb.setText(question);
    String query = "select text, answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' " +
                   "order by sequenceNo";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      while (rs.next()) {
	fb.setName(rs.getString(1));
	tmpAnswer.addElement(rs.getString(2));
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }
    fb.setCorrectAnswers(tmpAnswer);
    fb.setTime(timeLimit_min, timeLimit_sec); 
  }

  /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for matching
  *
  *******************************************************/
  public void getValues(Matching m, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    m.setText(question);
    String query = "select list, answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' ";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      while (rs.next()) {
	m.setPair(rs.getString(1), rs.getString(2));
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }
    m.setTime(timeLimit_min, timeLimit_sec);

  }

  /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for category
  *
  *******************************************************/
  public void getValues(Category c, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    c.setText(question);
    String query = "select list, answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' ";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      while (rs.next()) {
	c.setPair(rs.getString(1), rs.getString(2));
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }
    c.setTime(timeLimit_min, timeLimit_sec);
  }

 /*******************************************************
  *
  *  Method Name : getValues
  *  Function    : prepare the question for single choice
  *
  *******************************************************/
  public void getValues(Selection s, int questionID,
			 String question, int answerTypeNumber, 
			 int timeLimit_min, int timeLimit_sec, 
			 int totolTry, int correctAnswer, 
			 String relatedTable) {
    s.setText(question);
    String query = "select list, answer from " + relatedTable + " where " +
                   "questionID='" + questionID + "' ";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);

      while (rs.next()) {
	s.setOption(rs.getString(1),'s');
	if ( java.lang.Integer.parseInt(rs.getString(2)) == True ) {
	  s.setCorrectAnswer(rs.getString(1));
	}
      }

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    s.setTime(timeLimit_min, timeLimit_sec);
  }

  //
  // add data
  //
 /*******************************************************
  *
  *  Method Name : addAuthor
  *  Function    : add a new author
  *
  *******************************************************/
  public boolean addAuthor(String authorName) {

    String query = null;
    boolean done = true;
    int authorID = 0;

    // create authorID 
    query = "select authorID from authorTable";

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      int tmpID;
      while (rs.next()) {
	tmpID = java.lang.Integer.parseInt(rs.getString(1));
	if ( tmpID > authorID ) {
	  authorID = tmpID;
	}
      }
      authorID++;

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    printLine("add author");
    query = "insert into authorTable values (" +
            "'" + authorID + "', " +
            "'" + authorName + "'" +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : addCourse
  *  Function    : add a new course
  *
  *******************************************************/
  public boolean addCourse(String authorName, String courseName) {

    String query = null;
    boolean done = true;
    int authorID;
    int courseID = -1;

    // create courseID
    courseID = getCourseID(courseName);

    if ( courseID == -1 ) {
      query = "select courseID from courseTable";

      try {
	Statement sqlstmt = conn.createStatement();
	ResultSet rs = sqlstmt.executeQuery(query);
	int tmpID;
	courseID = 0;
	while (rs.next()) {
	  tmpID = java.lang.Integer.parseInt(rs.getString(1));
	  if ( tmpID > courseID ) {
	    courseID = tmpID;
	  }
	}
	courseID++;

	query = "insert into courseTable values (" +
	        "'" + courseID   + "', " +
                "'" + courseName + "'"   +
                ")";
	sqlstmt = conn.createStatement();
	sqlstmt.executeUpdate(query);

	rs.close();
	sqlstmt.close();
      }
      catch (Exception e) {
	printLine("Error in dbase");
	e.printStackTrace();
      }
    }

    authorID = getAuthorID(authorName);

    printLine("add course");
    query = "insert into authorCourse values (" +
            "'" + authorID + "', " +
            "'" + courseID + "'"   +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : addLecture
  *  Function    : add a new lecture
  *
  *******************************************************/
  public boolean addLecture(String authorName, String courseName, 
			    String lectureName) {

    String query = null;
    boolean done = true;
    int authorID;
    int courseID;
    int lectureID = -1;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);

    query = "select lectureID from lectureTable";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      int tmpID;
      lectureID = 0;
      while (rs.next()) {
	tmpID = java.lang.Integer.parseInt(rs.getString(1));
	if ( tmpID > lectureID ) {
	  lectureID = tmpID;
	}
      }
      lectureID++;

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    printLine("add lecture");
    query = "insert into lectureTable values (" +
            "'" + authorID    + "', " +
            "'" + courseID    + "', " +
            "'" + lectureID   + "',"  +
            "'" + lectureName + "'"   +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : addQuiz
  *  Function    : add a new quiz
  *
  *******************************************************/
  public boolean addQuiz(String authorName, String courseName, 
			 String lectureName, String quizName) {

    String query = null;
    boolean done = true;
    int authorID;
    int courseID;
    int lectureID;
    int quizID = -1;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);

    query = "select quizID from quizTable";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      int tmpID;
      quizID = 0;
      while (rs.next()) {
	tmpID = java.lang.Integer.parseInt(rs.getString(1));
	if ( tmpID > quizID ) {
	  quizID = tmpID;
	}
      }
      quizID++;

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    printLine("add quiz");
    query = "insert into quizTable values (" +
            "'" + authorID  + "', " +
            "'" + courseID  + "', " +
            "'" + lectureID + "', " +
            "'" + quizID    + "', " +
            "'" + quizName  + "'  " +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : addQuestion
  *  Function    : add a new question
  *
  *******************************************************/
  public boolean addQuestion(String authorName,
			     AbstractQuestion q) {
    String query = null;
    boolean done = true;
    int answerListLength;

    int authorID;
    int questionID = -1;
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = null;
    Vector answerList = new Vector();

    authorID = getAuthorID(authorName);

    query = "select questionID from questionTable";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      int tmpID;
      questionID = 0;
      while (rs.next()) {
	tmpID = java.lang.Integer.parseInt(rs.getString(1));
	if ( tmpID > questionID ) {
	  questionID = tmpID;
	}
      }
      questionID++;

      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
    }

    printLine("add question");

    if ( q instanceof TrueFalse ) {
      done = setValues(authorID, questionID, (TrueFalse) q);
    }
    else if ( q instanceof MultipleChoice ) {
      setValues(authorID, questionID, (MultipleChoice) q);
    }
    else if ( q instanceof FillInTheBlank ) {
      done = setValues(authorID, questionID, (FillInTheBlank) q);
    }
    else if ( q instanceof Matching ) {
      done = setValues(authorID, questionID, (Matching) q);
    }
    else if ( q instanceof Category ) {
      done = setValues(authorID, questionID, (Category) q);
    }
    else if ( q instanceof Selection ) {
      done = setValues(authorID, questionID, (Selection) q);
    }
    else {
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for true false
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   TrueFalse q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_trueFalse";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    String tmpAnswerType = q.getTrueText();
    String tmpAnswer = null;

    questionType = TRUE_FALSE;
    relatedTable = "q_trueFalse";
    if ( tmpAnswerType.equals(YES1) ||
	 tmpAnswerType.equals(YES2) ||
	 tmpAnswerType.equals(YES3) ) {
      answerTypeNumber = 1;
    }
    else if ( tmpAnswerType.equals(TRUE1) ||
	      tmpAnswerType.equals(TRUE2) ||
	      tmpAnswerType.equals(TRUE3) ) {
      answerTypeNumber = 2;
    }
    else {
      answerTypeNumber = 2;
    }

    if ( q.getCorrectAnswer() ) {
      tmpAnswer = "1";
    }
    else {
      tmpAnswer = "0";
    }
    answerList.addElement(tmpAnswer);

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID,  
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for multiple choice
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   MultipleChoice q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_multipleChoice";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    questionType = MULTIPLE_CHOICE;
    relatedTable = "q_multipleChoice";

    Vector tmpList = new Vector();
    Vector tmpAnswer = new Vector();

    tmpList = q.getOptions();
    tmpAnswer = q.getCorrectAnswer();

    answerTypeNumber = tmpAnswer.size();

    for ( int i = 0 ; i < tmpList.size() ; i++ ) {
      String answer = null;
      answerList.addElement(tmpList.elementAt(i));
      String tmpElement = (String)tmpList.elementAt(i);
      if ( tmpAnswer.contains(tmpElement) ) {
	answer = "1";
      }
      else {
	answer = "0";
      }
      answerList.addElement(answer);
    }

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID,  
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for fill in
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   FillInTheBlank q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_fillin";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    questionType = FILL_IN;
    relatedTable = "q_fillin";

    Vector tmpText = new Vector();
    Vector tmpAnswer = new Vector();

    tmpText = q.getNames();
    tmpAnswer = q.getCorrectAnswer();

    if ( tmpText.size() != tmpAnswer.size() ) {
      printLine("error - addQuestion (fill in)");
    }

    answerTypeNumber = tmpText.size();

    for ( int i = 0 ; i < tmpText.size() ; i++ ) {
      answerList.addElement(tmpText.elementAt(i));
      answerList.addElement(tmpAnswer.elementAt(i));
    }

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID,  
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for matching
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   Matching q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_matching";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    questionType = MATCHING;
    relatedTable = "q_matching";

    Vector tmpList = new Vector();
    Vector tmpAnswer = new Vector();

    tmpList = q.getItems();
    tmpAnswer = q.getCorrectAnswer();

    answerTypeNumber = tmpList.size();

    if ( tmpList.size() != tmpAnswer.size() ) {
      printLine("error - addQuestion (matching)");
    }

    for ( int i = 0 ; i < tmpList.size() ; i++ ) {
      answerList.addElement(tmpList.elementAt(i));
      answerList.addElement(tmpAnswer.elementAt(i));
    }

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID, 
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for category
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   Category q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_category";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    questionType = CATEGORY;
    relatedTable = "q_category";

    Vector tmpList = new Vector();
    Vector tmpAnswer = new Vector();

    tmpList = q.getItems();
    tmpAnswer = q.getCorrectAnswer();

    answerTypeNumber = tmpList.size();

    if ( tmpList.size() != tmpAnswer.size() ) {
      printLine("error - addQuestion (matching)");
    }

    for ( int i = 0 ; i < tmpList.size() ; i++ ) {
      answerList.addElement(tmpList.elementAt(i));
      answerList.addElement(tmpAnswer.elementAt(i));
    }

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID,  
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : setValues
  *  Function    : set values for single choice
  *
  *******************************************************/
  public boolean setValues(int authorID, int questionID,
			   Selection q) {
    String question = null;
    int questionType;
    int answerTypeNumber;
    int timeLimit_min;
    int timeLimit_sec;
    String relatedTable = "q_singleChoice";
    Vector answerList = new Vector();
    boolean done;

    question = q.getText();
    
    timeLimit_min = q.getMinutes();
    timeLimit_sec = q.getSeconds();

    questionType = SINGLE_CHOICE;
    relatedTable = "q_singleChoice";

    Vector tmpList = new Vector();
    String tmpAnswer = null;

    tmpList = q.getOptions();
    tmpAnswer = q.getCorrectAnswer();

    answerTypeNumber = 1;

    for ( int i = 0 ; i < tmpList.size() ; i++ ) {
      String answer = null;
      answerList.addElement(tmpList.elementAt(i));
      String tmpElement = (String)tmpList.elementAt(i);
      if ( tmpElement.equals(tmpAnswer) ) {
	answer = "1";
      }
      else {
	answer = "0";
      }
      answerList.addElement(answer);
    }

    done = saveQuestionCommon(authorID,
			      questionID,
			      question,
			      questionType,
			      answerTypeNumber,
			      timeLimit_min,
			      timeLimit_sec,
			      relatedTable);
    if ( done ) {
      done = saveRelatedTable(q, questionID,  
			      relatedTable, answerList);
    }
    else {
      return done;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveQuestionCommon
  *  Function    : save the question - common part
  *
  *******************************************************/
  public boolean saveQuestionCommon(int authorID,
				   int questionID,
				   String question,
				   int questionType,
				   int answerTypeNumber,
				   int timeLimit_min,
				   int timeLimit_sec,
				   String relatedTable) {
    String query = null;
    boolean done = true;

    query = "insert into questionTable values (" +
            "'" + authorID         + "', " +
            "'" + questionID       + "', " +
            "'" + question         + "', " +
            "'" + questionType     + "', " +
            "'" + answerTypeNumber + "', " +
            "'" + timeLimit_min    + "', " +
            "'" + timeLimit_sec    + "', " +
            "'0', " +
            "'0', " +
            "'" + relatedTable     + "'"   +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - true false related table
  *
  *******************************************************/
  public boolean saveRelatedTable(TrueFalse q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    int answer;
    String query = null;
    boolean done = true;

    answer = java.lang.Integer.parseInt((String)answerList.elementAt(0));
    query = "insert into " +
            relatedTable + " values (" +
            "'" + questionID + "', " +
            "'" + answer     + "'" +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - multiple choice related table
  *
  *******************************************************/
  public boolean saveRelatedTable(MultipleChoice q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    int answer;
    String query = null;
    String list = null;
    boolean done = true;

    try {
      Statement sqlstmt = conn.createStatement();
      for( int i = 0 ; i < answerList.size() / 2 ; i++) {
	list = (String)answerList.elementAt(2*i);
	answer = java.lang.Integer.parseInt((String)answerList.elementAt(2*i+1));
	query = "insert into " +
	        relatedTable + " values (" +
	        "'" + questionID + "', " +
	        "'" + list       + "', " +
	        "'" + answer     + "'"   +
	        ")";

	sqlstmt.executeUpdate(query);
      }
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - fill in related table
  *
  *******************************************************/
  public boolean saveRelatedTable(FillInTheBlank q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    String answer = null;
    String query = null;
    String list = null;
    boolean done = true;

    try {
      Statement sqlstmt = conn.createStatement();
      for( int i = 0 ; i < answerList.size() / 2 ; i++) {
	list = (String)answerList.elementAt(2*i);
	answer = (String)answerList.elementAt(2*i+1);
	query = "insert into " +
	        relatedTable + " values (" +
  	        "'" + questionID + "', " +
	        "'" + list       + "', " +
	        "'" + answer     + "', " +
	        "'" + i          + "'"   +
	        ")";

	sqlstmt.executeUpdate(query);
      }
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - matching related table
  *
  *******************************************************/
  public boolean saveRelatedTable(Matching q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    String answer = null;
    String query = null;
    String list = null;
    boolean done = true;

    try {
      Statement sqlstmt = conn.createStatement();
      for( int i = 0 ; i < answerList.size() / 2 ; i++) {
	list = (String)answerList.elementAt(2*i);
	answer = (String)answerList.elementAt(2*i+1);
	query = "insert into " +
	        relatedTable + " values (" +
  	        "'" + questionID + "', " +
	        "'" + list       + "', " +
	        "'" + answer     + "'"   +
	        ")";

	sqlstmt.executeUpdate(query);
      }
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - category related table
  *
  *******************************************************/
  public boolean saveRelatedTable(Category q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    String answer = null;
    String query = null;
    String list = null;
    boolean done = true;

    try {
      Statement sqlstmt = conn.createStatement();
      for( int i = 0 ; i < answerList.size() / 2 ; i++) {
	list = (String)answerList.elementAt(2*i);
	answer = (String)answerList.elementAt(2*i+1);
	query = "insert into " +
	        relatedTable + " values (" +
  	        "'" + questionID + "', " +
	        "'" + list       + "', " +
	        "'" + answer     + "'"   +
	        ")";

	sqlstmt.executeUpdate(query);
      }
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : saveRelatedTable
  *  Function    : save the question 
  *                - single choice related table
  *
  *******************************************************/
  public boolean saveRelatedTable(Selection q,
				  int questionID,
				  String relatedTable,
				  Vector answerList) {
    int answer;
    String query = null;
    String list = null;
    boolean done = true;

    try {
      Statement sqlstmt = conn.createStatement();
      for( int i = 0 ; i < answerList.size() / 2 ; i++) {
	list = (String)answerList.elementAt(2*i);
	answer = java.lang.Integer.parseInt((String)answerList.elementAt(2*i+1));
	query = "insert into " +
	        relatedTable + " values (" +
	        "'" + questionID + "', " +
	        "'" + list       + "', " +
	        "'" + answer     + "'"   +
	        ")";

	sqlstmt.executeUpdate(query);
      }
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }
    return done;
  }

 /*******************************************************
  *
  *  Method Name : addQuestion2Quiz
  *  Function    : add a question to a quiz
  *
  *******************************************************/
  public boolean addQuestion2Quiz(String authorName,
				  String courseName,
				  String lectureName,
				  String quizName, 
				  String question) {
    int authorID;
    int courseID;
    int lectureID;
    int quizID;
    int questionID;
    boolean done = true;
    String query = null;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);
    quizID = getQuizID(authorName, courseName, lectureName, quizName);
    questionID = getQuestionID(authorName, question);

    printLine("add question to quiz");
    query = "insert into quizQuestion values (" +
            "'" + quizID     + "', " +
            "'" + questionID + "'"   +
            ")";
    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;

  }

  //
  // remove data
  //
 /*******************************************************
  *
  *  Method Name : removeAuthor
  *  Function    : remove the author from
  *                - authorTable
  *                - authorCourse
  *                - lectureTable
  *                - quizTable
  *                - quizQuestion
  *                - questionTable
  *
  *******************************************************/
  public boolean removeAuthor(String authorName) {

    String queryAuthor = null;
    String queryAuthorCourse = null;
    String queryLecture = null;
    String queryQuiz = null;
    String queryQuestion = null;

    int authorID;
    boolean done = true;

    authorID = getAuthorID(authorName);

    queryAuthor = "delete from authorTable where " +
                  "authorID='" + authorID + "'";

    queryAuthorCourse = "delete from authorCourse where " +
                        "authorID='" + authorID + "'";

    queryLecture = "delete from lectureTable where " +
                   "authorID='" + authorID + "'";

    queryQuiz = "select courseID, lectureID, quizID from quizTable where " +
                 "authorID='" + authorID + "'";

    queryQuestion = "select questionID from questionTable where " +
                    "authorID='" + authorID + "'";

    printLine("remove author");

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(queryQuiz);
      while (rs.next()) {
	int courseID  = java.lang.Integer.parseInt(rs.getString(1));
	int lectureID = java.lang.Integer.parseInt(rs.getString(2));
	int quizID    = java.lang.Integer.parseInt(rs.getString(3));
	removeQuiz(authorName, courseID, 
		   lectureID, quizID);
      }
      rs.close();

      rs = sqlstmt.executeQuery(queryQuestion);
      while (rs.next()) {
	int questionID  = java.lang.Integer.parseInt(rs.getString(1));
	removeQuestion(authorID, questionID);
      }
      rs.close();
      sqlstmt.executeUpdate(queryAuthor);
      sqlstmt.executeUpdate(queryAuthorCourse);
      sqlstmt.executeUpdate(queryLecture);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeCourse
  *  Function    : remove the course from
  *                - authorCourse
  *                - courseTable
  *                - lectureTable
  *                - quizTable
  *                - quizQuestion
  *
  *******************************************************/
  public boolean removeCourse(String authorName, String courseName) {

    String queryAuthorCourse = null;
    String queryLecture = null;
    String queryQuiz = null;

    int authorID;
    int courseID;
    boolean done = true;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);

    queryAuthorCourse = "delete from authorCourse where " +
                        "courseID='" + courseID + "'";

    queryLecture = "delete from lectureTable where " +
                   "courseID='" + courseID + "'";

    queryQuiz = "select lectureID, quizID from quizTable where " +
                 "authorID='" + authorID + "' and " +
                 "courseID='" + courseID + "'";

    printLine("remove course");

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(queryQuiz);
      while (rs.next()) {
	int lectureID = java.lang.Integer.parseInt(rs.getString(1));
	int quizID    = java.lang.Integer.parseInt(rs.getString(2));
	removeQuiz(authorName, courseName, 
		   lectureID, quizID);
      }
      rs.close();
      sqlstmt.executeUpdate(queryAuthorCourse);
      sqlstmt.executeUpdate(queryLecture);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeLecture
  *  Function    : remove the lecture from
  *                - lectureTable
  *                - quizTable
  *                - quizQuestion
  *
  *******************************************************/
  public boolean removeLecture(String authorName, String courseName, 
			       String lectureName) {

    String queryLecture = null;
    String queryQuiz = null;
    int authorID;
    int courseID;
    int lectureID;
    int quizID;
    boolean done = true;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);

    queryLecture = "delete from lectureTable where " +
                   "authorID="  + "'" + authorID  + "' and " +
                   "courseID="  + "'" + courseID  + "' and " +
                   "lectureID=" + "'" + lectureID + "'";

    queryQuiz = "select quizName from quizTable where " +
                "authorID="  + "'" + authorID  + "' and " +
                "courseID="  + "'" + courseID  + "' and " +
                "lectureID=" + "'" + lectureID + "'";

    printLine("remove lecture");

    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(queryQuiz);
      while (rs.next()) {
	String quizName = rs.getString(1);
	removeQuiz(authorName, courseName, 
		   lectureName, quizName);
      }
      rs.close();
      sqlstmt.executeUpdate(queryLecture);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuiz
  *  Function    : remove the quiz from
  *                - quizTable
  *                - quizQuestion
  *
  *******************************************************/
  public boolean removeQuiz(String authorName, String courseName, 
			    String lectureName, String quizName) {

    String queryQuiz = null;
    String queryQuizQuestion = null;
    int authorID;
    int courseID;
    int lectureID;
    int quizID;
    boolean done = true;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);
    quizID = getQuizID(authorName, courseName, 
		       lectureName, quizName);

    queryQuiz = "delete from quizTable where " +
                "authorID="  + "'" + authorID  + "' and " +
                "courseID="  + "'" + courseID  + "' and " +
                "lectureID=" + "'" + lectureID + "' and " +
                "quizID="    + "'" + quizID    + "'";

    queryQuizQuestion = "delete from quizQuestion where " +
                        "quizID=" + "'" + quizID + "'";

    printLine("remove quiz");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(queryQuiz);
      sqlstmt.executeUpdate(queryQuizQuestion);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuiz
  *  Function    : remove the quiz from
  *                - quizTable
  *                - quizQuestion
  *
  *******************************************************/
  public boolean removeQuiz(String authorName, String courseName, 
			    int lectureID, int quizID) {

    String queryQuiz = null;
    String queryQuizQuestion = null;
    int authorID;
    int courseID;
    boolean done = true;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);

    queryQuiz = "delete from quizTable where " +
                "authorID="  + "'" + authorID  + "' and " +
                "courseID="  + "'" + courseID  + "' and " +
                "lectureID=" + "'" + lectureID + "' and " +
                "quizID="    + "'" + quizID    + "'";

    queryQuizQuestion = "delete from quizQuestion where " +
                        "quizID=" + "'" + quizID + "'";

    printLine("remove quiz - 2");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(queryQuiz);
      sqlstmt.executeUpdate(queryQuizQuestion);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuiz
  *  Function    : remove the quiz from
  *                - quizTable
  *                - quizQuestion
  *
  *******************************************************/
  public boolean removeQuiz(String authorName, int courseID, 
			    int lectureID, int quizID) {

    String queryQuiz = null;
    String queryQuizQuestion = null;
    int authorID;
    boolean done = true;

    authorID = getAuthorID(authorName);

    queryQuiz = "delete from quizTable where " +
                "authorID="  + "'" + authorID  + "' and " +
                "courseID="  + "'" + courseID  + "' and " +
                "lectureID=" + "'" + lectureID + "' and " +
                "quizID="    + "'" + quizID    + "'";

    queryQuizQuestion = "delete from quizQuestion where " +
                        "quizID=" + "'" + quizID + "'";

    printLine("remove quiz");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(queryQuiz);
      sqlstmt.executeUpdate(queryQuizQuestion);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuestion
  *  Function    : remove the question from
  *                - quizQuestion
  *                - questionTable
  *                - one of
  *                         o q_trueFalse
  *                         o q_singleChoice
  *                         o q_multipleChoice
  *                         o q_fillin
  *                         o q_matching
  *                         o q_category
  *
  *******************************************************/
  public boolean removeQuestion(String authorName, 
				String questionName) {

    String queryQuestion = null;
    String queryQuizQuestion = null;
    String queryQ1 = null;
    String queryQ2 = null;
    String queryQ3 = null;
    String queryQ4 = null;
    String queryQ5 = null;
    String queryQ6 = null;
    boolean done = true;
    int authorID;
    int questionID;

    authorID = getAuthorID(authorName);
    questionID = getQuestionID(authorName, questionName);
    
    queryQuestion = "delete from questionTable where " +
                    "authorID='"   + authorID   + "' and " +
                    "questionID='" + questionID + "'";

    queryQuizQuestion = "delete from quizQuestion where " +
                        "questionID='" + questionID + "'";

    queryQ1 = "delete from q_trueFalse where " +
              "questionID='" + questionID + "'";

    queryQ2 = "delete from q_singleChoice where " +
              "questionID='" + questionID + "'";

    queryQ3 = "delete from q_multipleChoice where " +
              "questionID='" + questionID + "'";

    queryQ4 = "delete from q_fillin where " +
              "questionID='" + questionID + "'";

    queryQ5 = "delete from q_matching where " +
              "questionID='" + questionID + "'";

    queryQ6 = "delete from q_category where " +
              "questionID='" + questionID + "'";

    printLine("remove question");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(queryQuestion);
      sqlstmt.executeUpdate(queryQuizQuestion);
      sqlstmt.executeUpdate(queryQ1);
      sqlstmt.executeUpdate(queryQ2);
      sqlstmt.executeUpdate(queryQ3);
      sqlstmt.executeUpdate(queryQ4);
      sqlstmt.executeUpdate(queryQ5);
      sqlstmt.executeUpdate(queryQ6);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuestion
  *  Function    : remove the question from
  *                - quizQuestion
  *                - questionTable
  *                - one of
  *                         o q_trueFalse
  *                         o q_singleChoice
  *                         o q_multipleChoice
  *                         o q_fillin
  *                         o q_matching
  *                         o q_category
  *
  *******************************************************/
  public boolean removeQuestion(int authorID, 
				int questionID) {

    String queryQuestion = null;
    String queryQuizQuestion = null;
    String queryQ1 = null;
    String queryQ2 = null;
    String queryQ3 = null;
    String queryQ4 = null;
    String queryQ5 = null;
    String queryQ6 = null;
    boolean done = true;

    queryQuestion = "delete from questionTable where " +
                    "authorID='"   + authorID   + "' and " +
                    "questionID='" + questionID + "'";

    queryQuizQuestion = "delete from quizQuestion where " +
                        "questionID='" + questionID + "'";

    queryQ1 = "delete from q_trueFalse where " +
              "questionID='" + questionID + "'";

    queryQ2 = "delete from q_singleChoice where " +
              "questionID='" + questionID + "'";

    queryQ3 = "delete from q_multipleChoice where " +
              "questionID='" + questionID + "'";

    queryQ4 = "delete from q_fillin where " +
              "questionID='" + questionID + "'";

    queryQ5 = "delete from q_matching where " +
              "questionID='" + questionID + "'";

    queryQ6 = "delete from q_category where " +
              "questionID='" + questionID + "'";

    printLine("remove question");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(queryQuestion);
      sqlstmt.executeUpdate(queryQuizQuestion);
      sqlstmt.executeUpdate(queryQ1);
      sqlstmt.executeUpdate(queryQ2);
      sqlstmt.executeUpdate(queryQ3);
      sqlstmt.executeUpdate(queryQ4);
      sqlstmt.executeUpdate(queryQ5);
      sqlstmt.executeUpdate(queryQ6);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : removeQuestionFromQuiz
  *  Function    : remove the question from quiz
  *
  *******************************************************/
  public boolean removeQuestionFromQuiz(String authorName, 
					String courseName, 
					String lectureName, 
					String quizName,
					String questionName) {

    int authorID;
    int courseID;
    int lectureID;
    int quizID;
    int questionID;
    String query = null;
    boolean done = true;

    quizID = getQuizID(authorName, courseName, 
		       lectureName, quizName);
    questionID = getQuestionID(authorName, questionName);

    query = "delete from quizQuestion where " +
            "quizID='"     + quizID + "' and " +
            "questionID='" + questionID + "'";

    printLine("remove question from quiz");

    try {
      Statement sqlstmt = conn.createStatement();
      sqlstmt.executeUpdate(query);
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("Error in dbase");
      e.printStackTrace();
      done = false;
    }

    return done;
  }

 /*******************************************************
  *
  *  Method Name : getAuthorID
  *  Function    : get an author ID from its name
  *
  *******************************************************/
  int getAuthorID(String authorName) {
    String query = null;
    int authorID = 0;

    query = "select authorID from authorTable where " +
            "authorName='" + authorName + "'";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( rs.next() ) {
	authorID = java.lang.Integer.parseInt(rs.getString(1));
      }
      else {
	authorID = -1;
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("DB error in getAuthorID");
    }
    return authorID;
  }

 /*******************************************************
  *
  *  Method Name : getCourseID
  *  Function    : get an course ID from its name
  *
  *******************************************************/
  int getCourseID(String courseName) {
    String query = null;
    int courseID = 0;

    query = "select courseID from courseTable where " +
            "courseName='" + courseName + "' ";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( rs.next() ) {
	courseID = java.lang.Integer.parseInt(rs.getString(1));
      }
      else {
	courseID = -1;
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("DB error in getCourseID");
    }
    return courseID;
  }

 /*******************************************************
  *
  *  Method Name : getLectureID
  *  Function    : get an lecture ID from its name
  *
  *******************************************************/
  int getLectureID(String authorName, String courseName, 
		   String lectureName) {
    String query = null;
    int authorID;
    int courseID;
    int lectureID = 0;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);

    query = "select lectureID from lectureTable where " +
            "authorID='" + authorID + "' and " +
            "courseID='" + courseID + "' and " +
            "lectureName='" + lectureName + "' ";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( rs.next() ) {
	lectureID = java.lang.Integer.parseInt(rs.getString(1));
      }
      else {
	lectureID = -1;
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("DB error in getLectureID");
    }
    return lectureID;
  }

 /*******************************************************
  *
  *  Method Name : getQuizID
  *  Function    : get an quiz ID from its name
  *
  *******************************************************/
  int getQuizID(String authorName, String courseName, 
		String lectureName, String quizName) {
    String query = null;
    int authorID;
    int courseID;
    int lectureID;
    int quizID = 0;

    authorID = getAuthorID(authorName);
    courseID = getCourseID(courseName);
    lectureID = getLectureID(authorName, courseName, lectureName);

    query = "select quizID from quizTable where " +
            "authorID='" + authorID + "' and " +
            "courseID='" + courseID + "' and " +
            "lectureID='" + lectureID + "' and " +
            "quizName='" + quizName + "'";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( rs.next() ) {
	quizID = java.lang.Integer.parseInt(rs.getString(1));
      }
      else {
	quizID = -1;
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("DB error in getQuizID");
    }
    return quizID;
  }

 /*******************************************************
  *
  *  Method Name : getQuestionID
  *  Function    : get an question ID from its name
  *
  *******************************************************/
  int getQuestionID(String authorName, String questionName) {
    String query = null;
    int authorID;
    int questionID = 0;

    authorID = getAuthorID(authorName);

    query = "select questionID from questionTable where " +
            "authorID='" + authorID     + "' and " +
            "question='" + questionName + "'";
    try {
      Statement sqlstmt = conn.createStatement();
      ResultSet rs = sqlstmt.executeQuery(query);
      if ( rs.next() ) {
	questionID = java.lang.Integer.parseInt(rs.getString(1));
      }
      else {
	questionID = -1;
      }
      rs.close();
      sqlstmt.close();
    }
    catch (Exception e) {
      printLine("DB error in getQuestionID");
    }
    return questionID;
  }

 /*******************************************************
  *
  *  Method Name : 
  *  Function    :
  *
  *******************************************************/
  public static void main(String args[]) {
    DBAccess dbaccess = new DBAccess();

    Vector tmpVector = new Vector();

    QuestionFactory questionFactory = new QuestionFactory();
    TrueFalse tf = (TrueFalse) questionFactory.create(QuestionFactory.TRUEFALSE);
    MultipleChoice mc = (MultipleChoice) questionFactory.create(QuestionFactory.MULTIPLECHOICE);
    FillInTheBlank fb = (FillInTheBlank) questionFactory.create(QuestionFactory.FILLINTHEBLANK);
    Matching m = (Matching) questionFactory.create(QuestionFactory.MATCHING);
    Category c = (Category) questionFactory.create(QuestionFactory.CATEGORY);
    Selection s = (Selection) questionFactory.create(QuestionFactory.SELECTION);

    /**
    TrueFalse tf = new TrueFalse();
    MultipleChoice mc = new MultipleChoice();
    FillInTheBlank fb = new FillInTheBlank();
    Matching m = new Matching();
    Category c = new Category();
    Selection s = new Selection();
    **/

    // open the database
    dbaccess.open();

    /**/
    // example add data
    dbaccess.addAuthor("Geoffrey Fox");
    dbaccess.addAuthor("Marek Podgorny");
    dbaccess.addAuthor("Nancy McCracken");

    dbaccess.addCourse("Geoffrey Fox", "CPS615");
    dbaccess.addCourse("Geoffrey Fox", "CPS616");
    dbaccess.addCourse("Geoffrey Fox", "CPS714");
    dbaccess.addCourse("Marek Podgorny", "CPS640");
    dbaccess.addCourse("Nancy McCracken", "CPS606");

    dbaccess.addLecture("Geoffrey Fox", "CPS615", "Introduction");
    dbaccess.addLecture("Geoffrey Fox", "CPS615", "Laplace Example");
    dbaccess.addLecture("Geoffrey Fox", "CPS615", "MPI Message Passing Interface");
    dbaccess.addLecture("Geoffrey Fox", "CPS615", "N Body Problem");

    dbaccess.addLecture("Geoffrey Fox", "CPS616", "Introduction");
    dbaccess.addLecture("Geoffrey Fox", "CPS616", "Database");
    dbaccess.addLecture("Geoffrey Fox", "CPS616", "JDBC");
    dbaccess.addLecture("Geoffrey Fox", "CPS616", "Java Servlet");
    dbaccess.addLecture("Geoffrey Fox", "CPS616", "Java RMI");
    dbaccess.addLecture("Geoffrey Fox", "CPS616", "CORBA");

    dbaccess.addLecture("Geoffrey Fox", "CPS714", "Case Studies in Internetics - Introductory Material");
    dbaccess.addLecture("Geoffrey Fox", "CPS714", "Introduction to Portals");

    dbaccess.addLecture("Marek Podgorny", "CPS640", "Nerworking: Introduction");
    dbaccess.addLecture("Marek Podgorny", "CPS640", "LAN Technology");
    dbaccess.addLecture("Marek Podgorny", "CPS640", "WAN Technology");
    dbaccess.addLecture("Marek Podgorny", "CPS640", "Routing");
    dbaccess.addLecture("Marek Podgorny", "CPS640", "Basic protocol");

    dbaccess.addLecture("Nancy McCracken", "CPS606", "Introduction to Java Language");
    dbaccess.addLecture("Nancy McCracken", "CPS606", "Java Language and Object-oriented Concepts");
    dbaccess.addLecture("Nancy McCracken", "CPS606", "Graphics and the Abstract Windowing Toolkit");
    dbaccess.addLecture("Nancy McCracken", "CPS606", "Multithreading, useful Java classes, I/O, and Networking");
    dbaccess.addLecture("Nancy McCracken", "CPS606", "Java Remote Method Invocation");

    dbaccess.addQuiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction");

    // true false
    tf.setText("RMI uses Java Language as IDL language");
    tf.setTrueText("True");
    tf.setFalseText("False");
    tf.setCorrectAnswer(true);
    tf.setTime(0, 30);
    dbaccess.addQuestion("Geoffrey Fox", tf);

    // multiple choice
    mc.setText("What are the advantages of the Distributed Memory Machine?");
    mc.setOption("Relatively easy to design and build", 'a');
    mc.setOption("Easy to program", 'a');
    mc.setOption("Program and O/S must be alone", 'a');
    mc.setOption("Can be scalable", 'a');
    mc.addCorrectAnswer("Relatively easy to design and build");
    mc.addCorrectAnswer("Can be scalable");
    mc.setTime(0, 40);
    dbaccess.addQuestion("Geoffrey Fox", mc);

    // fill in
    fb.setText("What does RMI stand for?");
    fb.setName("R");
    fb.setName("M");
    fb.setName("I");
    //tmpVector.clear();
    tmpVector.addElement("Remote");
    tmpVector.addElement("Method");
    tmpVector.addElement("Invocation");
    fb.setCorrectAnswers(tmpVector);
    fb.setTime(1, 50);
    dbaccess.addQuestion("Geoffrey Fox", fb);

    // matching
    m.setText("Match the related word");
    m.setPair("HTML", "document");
    m.setPair("VRML", "3D object");
    m.setPair("JDBC", "database connection");
    m.setTime(1, 55);
    dbaccess.addQuestion("Geoffrey Fox", m);

    // category
    c.setText("Parallel Language Matching");
    c.setPair("HPF", "Data Parallel Language");
    c.setPair("MPI", "Message Parallel Language");
    c.setPair("HPC++", "Data Parallel Language");
    c.setPair("PVM", "Message Parallel Language");
    c.setPair("Occam", "Message Parallel Language");
    c.setPair("C*", "Data Parallel Language");
    c.setTime(1, 15);
    dbaccess.addQuestion("Geoffrey Fox", c);

    // single choice
    s.setText("Which has the most capacity?");
    s.setOption("L2 Cache", 'a');
    s.setOption("L3 Cache", 'a');
    s.setOption("Main Memory", 'a');
    s.setOption("Disk", 'a');
    s.setCorrectAnswer("30");
    s.setTime(0, 45);
    dbaccess.addQuestion("Geoffrey Fox", s);

    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "RMI uses Java Language as IDL language");
    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "What are the advantages of the Distributed Memory Machine?");
    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "What does RMI stand for?");
    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "Match the related word");
    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "Parallel Language Matching");
    dbaccess.addQuestion2Quiz("Geoffrey Fox", "CPS615", "Introduction", "Quiz-Introduction", "Which has the most capacity?");
    /**/

    // example remove data

    //dbaccess.removeQuestionFromQuiz("Dongmin Kim", "kdm001", "kdm001-lec001", "kdm001-lecture001-quiz001", "Dongmin is a MALE");
    /**
    System.out.println("question list in quiz");
    dbaccess.removeQuestion("Dongmin Kim", "Dongmin is a MALE");
    tmpVector = dbaccess.getQuiz("Dongmin Kim", "kdm001", "kdm001-lec001", "kdm001-lecture001-quiz001");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }
    **/
    /**
    System.out.println("quiz list");
    dbaccess.removeQuiz("Dongmin Kim", "kdm001", "kdm001-lec001", "kdm001-lecture001-quiz001");
    tmpVector = dbaccess.getQuizList("Dongmin Kim", "kdm001", "kdm001-lec001");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }
    **/
    /**
    System.out.println("lecture list");
    dbaccess.removeLecture("Dongmin Kim", "kdm001", "kdm001-lec001");
    tmpVector = dbaccess.getLectureList("Dongmin Kim", "kdm001");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }
    **/

    /**
    System.out.println("course list");
    dbaccess.removeCourse("Dongmin Kim", "kdm001");
    tmpVector = dbaccess.getCourseList("Dongmin Kim");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }
    **/

    /**
    System.out.println("author list");
    dbaccess.removeAuthor("Dongmin Kim");
    tmpVector = dbaccess.getAuthorList();
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    **/
    /**/
    // example - retreive data
    tmpVector = dbaccess.getAuthorList();
    System.out.println("author list");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    tmpVector = dbaccess.getCourseList("Dongmin Kim");
    System.out.println("course list");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    tmpVector = dbaccess.getLectureList("Dongmin Kim", "kdm001");
    System.out.println("lecture list");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    tmpVector = dbaccess.getQuizList("Dongmin Kim", "kdm001", "kdm001-lec001");
    System.out.println("quiz list");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    tmpVector = dbaccess.getQuestionList("Dongmin Kim");
    System.out.println("total question list");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    tmpVector = dbaccess.getQuiz("Dongmin Kim", "kdm001", "kdm001-lec001", "kdm001-lecture001-quiz001");
    System.out.println("question list in quiz");
    for( int i = 0 ; i < tmpVector.size(); i++) {
      System.out.println("vector(" + i + ") = " + (String)tmpVector.elementAt(i));
    }

    System.out.println("question - true false");
    tf = (TrueFalse)dbaccess.getQuestion("Dongmin Kim", "Dongmin is a MALE");
    System.out.println("Question          : " + tf.getText());
    System.out.println("True Answer Type  : " + tf.getTrueText());
    System.out.println("False Answer Type : " + tf.getFalseText());
    System.out.println("Answer            : " + tf.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + tf.getMinutes());
    System.out.println("Time Limit (sec)  : " + tf.getSeconds());

    System.out.println("question - multiple choice");
    mc = (MultipleChoice)dbaccess.getQuestion("Dongmin Kim", "What are between 25 and 45 ?");
    System.out.println("Question          : " + mc.getText());
    System.out.println("Lists             : " + mc.getOptions());
    System.out.println("Answer            : " + mc.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + mc.getMinutes());
    System.out.println("Time Limit (sec)  : " + mc.getSeconds());

    System.out.println("question - fill in");
    fb = (FillInTheBlank)dbaccess.getQuestion("Dongmin Kim", "NPAC stands for");
    System.out.println("Question          : " + fb.getText());
    System.out.println("Texts             : " + fb.getNames());
    System.out.println("Answer            : " + fb.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + fb.getMinutes());
    System.out.println("Time Limit (sec)  : " + fb.getSeconds());

    System.out.println("question - matching");
    m = (Matching)dbaccess.getQuestion("Dongmin Kim", "First Name - Last Name match");
    System.out.println("Question          : " + m.getText());
    System.out.println("Lists             : " + m.getItems());
    System.out.println("Answer            : " + m.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + m.getMinutes());
    System.out.println("Time Limit (sec)  : " + m.getSeconds());

    System.out.println("question - category");
    c = (Category)dbaccess.getQuestion("Dongmin Kim", "Male-Female");
    System.out.println("Question          : " + c.getText());
    System.out.println("Lists             : " + c.getItems());
    System.out.println("Answer            : " + c.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + c.getMinutes());
    System.out.println("Time Limit (sec)  : " + c.getSeconds());



    System.out.println("question - single choice");
    s = (Selection)dbaccess.getQuestion("Dongmin Kim", "What is 5 X 6 ?");
    System.out.println("Question          : " + s.getText());
    System.out.println("Lists             : " + s.getOptions());
    System.out.println("Answer            : " + s.getCorrectAnswer());
    System.out.println("Time Limit (min)  : " + s.getMinutes());
    System.out.println("Time Limit (sec)  : " + s.getSeconds());

    /**/

    // close database
    dbaccess.close();
    System.out.println("done");
  }
}
